import React from "react";
import ReactDOM from "react-dom/client";
import App from "./App";
import "./index.css";
import registerAllEmailTriggers from "./lib/registerEmailTriggers";
import "./lib/googleMaps"; // Import Google Maps utility to ensure it loads properly
import { capacitorService } from "./services/capacitorService"; // Import Capacitor service

// Declare window properties for TypeScript
declare global {
  interface Window {
    hasFCP: boolean;
    hasStartedRendering: boolean;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    google?: any;
    googleMapsCallback?: () => void;
  }
}

// Add performance markers for debugging
if (process.env.NODE_ENV !== "production") {
  performance.mark("app-init-start");
}

// Initialize Capacitor for mobile apps
const initializeCapacitor = async () => {
  try {
    if (capacitorService.isNativePlatform()) {
      console.log(
        `🚀 Initializing Capacitor for ${capacitorService.getPlatform()}`
      );
      await capacitorService.initializeApp();
      console.log("✅ Capacitor initialized successfully");
    } else {
      console.log("🌐 Running in web mode");
    }
  } catch (error) {
    console.error("❌ Error initializing Capacitor:", error);
  }
};

// Set up error boundary for the entire application
const ErrorFallback = () => {
  return (
    <div
      className="error-container"
      style={{
        padding: "20px",
        margin: "20px",
        border: "1px solid #f5c6cb",
        borderRadius: "4px",
        color: "#721c24",
        backgroundColor: "#f8d7da",
      }}
    >
      <h2>Something went wrong</h2>
      <p>
        We're sorry, but there was an error loading the application. Please try
        refreshing the page.
      </p>
      <button
        onClick={() => window.location.reload()}
        style={{
          padding: "8px 16px",
          backgroundColor: "#721c24",
          color: "white",
          border: "none",
          borderRadius: "4px",
          cursor: "pointer",
        }}
      >
        Refresh
      </button>
    </div>
  );
};

// Register all email triggers when the app starts - move to a background task
setTimeout(() => {
  registerAllEmailTriggers().catch((error) => {
    console.error("Failed to register email triggers:", error);
  });
}, 3000);

// Mark FCP for performance monitoring
const markFCP = () => {
  window.hasFCP = true;
  if (process.env.NODE_ENV !== "production") {
    performance.mark("fcp-detected");
  }
};

// Initialize the application
const initializeApp = async () => {
  try {
    // Initialize Capacitor first for mobile platforms
    await initializeCapacitor();

    const rootElement = document.getElementById("root");
    if (!rootElement) throw new Error("Root element not found");

    const root = ReactDOM.createRoot(rootElement);

    // Create a MutationObserver to detect when content is added to the DOM
    const observer = new MutationObserver((mutations) => {
      if (mutations.some((m) => m.addedNodes.length > 0)) {
        markFCP();
        observer.disconnect();
      }
    });

    // Start observing
    observer.observe(rootElement, { childList: true, subtree: true });

    // Render the app
    root.render(
      <React.StrictMode>
        <React.Suspense fallback={<div>Loading...</div>}>
          <App />
        </React.Suspense>
      </React.StrictMode>
    );

    if (process.env.NODE_ENV !== "production") {
      performance.mark("app-init-end");
      performance.measure(
        "App Initialization",
        "app-init-start",
        "app-init-end"
      );
    }
  } catch (error) {
    console.error("Error initializing application:", error);
    const rootElement = document.getElementById("root");
    if (rootElement) {
      ReactDOM.createRoot(rootElement).render(<ErrorFallback />);
    }
  }
};

// Start the application
initializeApp();
